import { __assign } from "tslib";
import React, { useContext, useEffect, useMemo, useState } from 'react';
import { FeedManager } from './FeedManager';
import { useStreamContext } from './StreamApp';
export var FeedContext = React.createContext({});
export var FeedProvider = function (_a) {
    var children = _a.children, value = _a.value;
    return React.createElement(FeedContext.Provider, { value: value }, children);
};
export var useFeedContext = function () { return useContext(FeedContext); };
export function Feed(props) {
    var _a = useStreamContext(), analyticsClient = _a.analyticsClient, client = _a.client, user = _a.user, errorHandler = _a.errorHandler, sharedFeedManagers = _a.sharedFeedManagers;
    var feedGroup = props.feedGroup, userId = props.userId, children = props.children;
    var _b = useState(0), setForceUpdateState = _b[1];
    var manager = useMemo(function () {
        if (!client)
            return null;
        // const clientDifferent = this.props.client !== prevProps.client;
        // const notifyDifferent = this.props.notify !== prevProps.notify;
        // const feedDifferent = this.props.userId !== prevProps.userId || this.props.feedGroup !== prevProps.feedGroup;
        // const optionsDifferent = !_isEqual(this.props.options, prevProps.options);
        // if (clientDifferent || feedDifferent || optionsDifferent || notifyDifferent)
        //   TODO: Implement
        var feedId = client.feed(feedGroup, userId).id;
        return (sharedFeedManagers[feedId] ||
            new FeedManager(__assign(__assign({}, props), { analyticsClient: analyticsClient, client: client, user: user, errorHandler: errorHandler })));
    }, [feedGroup]);
    useEffect(function () {
        var forceUpdate = function () { return setForceUpdateState(function (prevState) { return prevState + 1; }); };
        manager === null || manager === void 0 ? void 0 : manager.register(forceUpdate);
        return function () { return manager === null || manager === void 0 ? void 0 : manager.unregister(forceUpdate); };
    }, [manager]);
    if (!manager)
        return null;
    var ctx = {
        feedGroup: feedGroup,
        userId: userId,
        feedManager: manager,
        getActivityPath: manager.getActivityPath,
        onToggleReaction: manager.onToggleReaction,
        onAddReaction: manager.onAddReaction,
        onRemoveReaction: manager.onRemoveReaction,
        onToggleChildReaction: manager.onToggleChildReaction,
        onAddChildReaction: manager.onAddChildReaction,
        onRemoveChildReaction: manager.onRemoveChildReaction,
        onRemoveActivity: manager.onRemoveActivity,
        onMarkAsRead: manager.onMarkAsRead,
        onMarkAsSeen: manager.onMarkAsSeen,
        refresh: manager.refresh,
        refreshUnreadUnseen: manager.refreshUnreadUnseen,
        loadNextReactions: manager.loadNextReactions,
        loadNextPage: manager.loadNextPage,
        hasNextPage: manager.hasNextPage(),
        loadReverseNextPage: manager.loadReverseNextPage,
        hasReverseNextPage: manager.hasReverseNextPage(),
        activityOrder: manager.state.activityOrder,
        activities: manager.state.activities,
        realtimeAdds: manager.state.realtimeAdds,
        realtimeDeletes: manager.state.realtimeDeletes,
        refreshing: manager.state.refreshing,
        unread: manager.state.unread,
        unseen: manager.state.unseen,
        hasDoneRequest: manager.state.lastResponse != null,
    };
    return React.createElement(FeedProvider, { value: ctx }, children);
}
//# sourceMappingURL=Feed.js.map